/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package com.notuvy.file;

import java.io.*;

import org.apache.log4j.Logger;

/**
 * A utility to save string content to a file, and to extract the string content from a file.
 *
 * @author  murali
 */
public class FileContent {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(FileContent.class);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final File fDestination;


    //------------------------------------------------------------
    //- Constructors

    public FileContent(File pParent, String pName) {
        fDestination = new File(pParent, pName);
    }


    public FileContent(File pDestination) {
        fDestination = pDestination;
    }


    //------------------------------------------------------------
    //- Accessors

    public File getDestination() {
        return fDestination;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public boolean fileExists() {
        return getDestination().exists();
    }


    public boolean save(Object pContent) {
        boolean success = false;  // Pessimistic.

        if (!getDestination().exists() || getDestination().isFile()) {
            try {
                FileWriter writer = new FileWriter(getDestination());
                writer.write(String.valueOf(pContent));
                writer.close();
                // LOG.info("Wrote [" + getDestination() + "]");
                success = true;
            } catch (FileNotFoundException fnfe) {
                LOG.error("No output file.", fnfe);
            } catch (IOException ioe) {
                LOG.error("Problem writing output file [" + getDestination() + "].", ioe);
            }
        } else {
            LOG.error("Not a plain file: " + this);
        }

        return success;
    }


    public String readString() {
        String result = "";

        if (getDestination().isFile()) {
            boolean wasTextFile = true;
            StringWriter writer = new StringWriter();
            try {
                wasTextFile = transferTo(writer);
                writer.close();
            } catch (IOException ioe) {
                LOG.error("Problem reading string output file [" + getDestination() + "].", ioe);
            }

            if(wasTextFile) {
                result = writer.toString();
            } else {
                LOG.error("Not a text file: " + this);
            }
        } else {
            LOG.error("Not a plain file: " + this);
        }

        return result;
    }


    public boolean transferTo(Writer pWriter) throws IOException {
        boolean allCharsAreValid = true;  // Optimistic
        try {
            char[] chunk = new char[1024];

            FileReader reader = new FileReader(getDestination());
            int numRead = reader.read(chunk);
            while (allCharsAreValid && (numRead != -1)) {
                for (int i=0; allCharsAreValid && i<numRead; i++) {
                    allCharsAreValid = Character.isDefined(chunk[i]);
                }
                pWriter.write(chunk, 0, numRead);
                numRead = reader.read(chunk);
            }
            reader.close();
            LOG.debug("Read [" + getDestination() + "]");
        } catch (FileNotFoundException fnfe) {
            LOG.error("No input file.", fnfe);
        }

        return allCharsAreValid;
    }


    public boolean readLines(LineHandler pHandler) {
        boolean success = false;  // Pessimistic.

        try {
            FileReader fr = new FileReader(getDestination());
            LineNumberReader lnr = new LineNumberReader(fr);

            for (String line=lnr.readLine(); line!=null; line=lnr.readLine()) {
                pHandler.handle(lnr.getLineNumber(), line);
            }

            lnr.close();
            fr.close();
            success = true;
        } catch (FileNotFoundException fnfe) {
            LOG.error("No input file.", fnfe);
        } catch (IOException ioe) {
            LOG.error("Problem reading input file.", ioe);
        }

        return success;
    }


    public boolean save(WriterHandler pHandler) {
        boolean success = false;  // Pessimistic.

        try {
            FileWriter writer = new FileWriter(getDestination());

            pHandler.handle(writer);

            writer.close();

            success = true;
        } catch (IOException ioe) {
            LOG.error("Could not save account data to [" + getDestination() + "].", ioe);
        }

        return success;
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public String toString() {
        return String.format("FileContent[%s]", getDestination().getAbsolutePath());
    }


    //------------------------------------------------------------
    //- Inner Classes

    public interface LineHandler {
        public void handle(int number, String line);
    }

    public interface WriterHandler {
        public void handle(FileWriter writer) throws IOException;
    }



    //------------------------------------------------------------
    //- Main



}
