package com.notuvy.util;

import org.apache.log4j.Logger;
import org.apache.commons.lang.StringUtils;

import java.util.regex.Pattern;
import java.util.LinkedList;
import java.util.List;
import java.util.Iterator;

/**
 * A comparable representation of a software product version.
 *
 * @author murali
 */
public class ProductVersion implements Comparable<ProductVersion> {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger("com.snipfolio.gui");

    private static final Pattern P_ZERO = Pattern.compile("^\\s*0+\\s*$");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final LinkedList<String> fParts = new LinkedList<String>();


    //------------------------------------------------------------
    //- Constructors

    private ProductVersion(List<String> pParts) {
        fParts.addAll(pParts);
    }

    public ProductVersion(String pString) {
        if (StringUtils.isNotBlank(pString)) {
            for (String part : pString.split("\\.")) {
                fParts.addLast(part.trim());
            }
        }
    }


    //------------------------------------------------------------
    //- Gettors

    public LinkedList<String> getParts() {
        return fParts;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private int compareToUnequal(ProductVersion pOther) {
        int result = major().compareTo(pOther.major());

        if (result == 0) {
            result = toNextMinor().compareToUnequal(pOther.toNextMinor());
        }

        return result;
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public String major() {
        return getParts().isEmpty() ? "0" : getParts().getFirst();
    }

    public ProductVersion toNextMinor() {
        ProductVersion result;
        if (getParts().isEmpty()) {
            result = this;
        } else {
            LinkedList<String> copy = new LinkedList<String>(getParts());
            copy.removeFirst();
            result = new ProductVersion(copy);
        }
        return result;
    }

    public ProductVersion toCanonical() {
        ProductVersion result = this;  // Assume already canonical.
        if (getParts().isEmpty()) {
            result = new ProductVersion("0");
        } else if (P_ZERO.matcher(getParts().getLast()).find()) {
            LinkedList<String> copy = new LinkedList<String>(getParts());
            while ((copy.size() > 1) && P_ZERO.matcher(copy.getLast()).find()) {
                copy.removeLast();
            }
            result = new ProductVersion(copy);
        }
        return result;
    }

    public boolean lessThan(ProductVersion pOther) {
        return compareTo(pOther) < 0;
    }

    public boolean greaterThan(ProductVersion pOther) {
        return compareTo(pOther) > 0;
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    @Override
    public boolean equals(Object pObject) {
        boolean result = false;
        if (pObject instanceof ProductVersion) {
            ProductVersion comp1 = toCanonical();
            ProductVersion comp2 = ((ProductVersion) pObject).toCanonical();
            if (comp1.getParts().size() == comp2.getParts().size()) {
                result = true; // Temporary setting.
                Iterator<String> iter1 = comp1.getParts().iterator();
                Iterator<String> iter2 = comp2.getParts().iterator();
                while (result && iter1.hasNext()) {
                    result = iter1.next().equals(iter2.next());
                }
            }
        }
        return result;
    }

    public int compareTo(ProductVersion pOther) {
        return equals(pOther) ? 0 : toCanonical().compareToUnequal(pOther.toCanonical());
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        if (getParts().isEmpty()) {
            builder.append("0");
        } else {
            for (String part : getParts()) {
                if (builder.length() > 0) {
                    builder.append(".");
                }
                builder.append(part);
            }
        }
        return builder.toString();
    }


    //------------------------------------------------------------
    //- Inner Classes


    //------------------------------------------------------------
    //- Main


}
